;
; Check scripts errors free
Opt("TrayIconDebug", 1)

#Region Includes
#include "include\OutputLib.au3"
#include "include\SettingsLib.au3"
#include "StringConstants.au3"
#EndRegion Includes

#Region Global Variables
; The name of the project.
Global Const $g_sProject = "Check Variable Names"
Global Const $g_sProjectDir = "install"

#Region Total Partial commenting
; Just uncomment with Ctrl-Q SciTE to check not all

;~ Global Const $g_sFiles = "Excel"

Global Const $g_sFiles = ""
;~ Global Const $g_aDirectories[1][2] = [ _
;~ ["install\Include\", True] _
;~ ]

Global Const $g_aDirectories[27][2] = [ _
		["install\Examples\", True], _
		["install\Examples\GUI\", True], _
		["install\Examples\GUI\Advanced\", True], _
		["install\Examples\GUI\Simple\", True], _
		["install\Examples\COM\", True], _
		["install\Extras\", True], _
		["install\Extras\AutoUpdateIt\", True], _
		["install\Extras\Editors\Crimson\", True], _
		["install\Extras\Editors\TextPad\", True], _
		["install\Include\", True], _
		["docs\autoit\english\txt2htm\examples\", True], _
		["docs\autoit\english\txt2htm\libExamples\", True], _
		["docs\autoit\english\txt2htm\libExamples\Extras\", True], _
		["docs\autoit\english\txt2htm\txtlibfunctions\", False], _
		["_build\Include\", True], _
		["_build\", True], _
		["docs\_build\", True], _
		["docs\_build\Include\", True], _
		["docs\_build\Include\Help Check\", True], _
		["docs\autoitx\english\", True], _
		["docs\autoit\english\", True], _
		["docs\autoit\english\txt2htm\syntaxfiles\Crimson\", True], _
		["docs\autoit\english\txt2htm\syntaxfiles\Geshi\", True], _
		["docs\autoit\english\txt2htm\syntaxfiles\Notepad++\", True], _
		["docs\autoit\english\txt2htm\syntaxfiles\PSPAD\", True], _
		["docs\autoit\english\txt2htm\syntaxfiles\Sublime Text\", True], _
		["docs\autoit\english\txt2htm\syntaxfiles\TextPad\", True] _
		]

; End uncommenting zone
#EndRegion Total Partial commenting

Global $g_bConst, $g_bGlobal, $g_bGUICtrlCreate, $g_sVarnameError, $g_iNbErrors ; to be used when checking convention
#EndRegion Global Variables

#Region Main body of code
Global $g_nExitCode = _Main($g_sProject)
Exit $g_nExitCode
#EndRegion Main body of code

#Region _Main()
; ===================================================================
; _Main($sProject)
;
; The main function responsible for archiving the directories.
; Parameters:
;	$sProject - IN - The name to display in the progress output.
; Returns:
;	0 on success, non-zero on failure.
; ===================================================================
Func _Main($sProject)
	; Create the output window and initial message.
	_OutputWindowCreate()
	_OutputProgressWrite("==== Output for " & StringTrimRight(@ScriptName, StringLen(".exe")) & " (" & $sProject & ") ====" & @CRLF)

	; Set the build directory based on the rules and the INI file value.
	_SettingBuildDir()

	; A list of directories with files to check.

	; Cache the directory count since it won't change at runtime.
	Local Const $nDirectoryCount = UBound($g_aDirectories)

	; Count the number of files successfully parsed.
	Local $nSuccessCount = 0
	Local $nFailedCount = 0

	Local $bResult, $iNbErrors, $iNbFiles, $hSearch

	; Check the files in each directory.
	For $i = 0 To $nDirectoryCount - 1
		; Output a message about which directory is being tested.
		_OutputProgressWrite("(" & $i + 1 & "/" & $nDirectoryCount & ") " & 'Testing files in "' & $g_aDirectories[$i][0] & '"... ')

		; Ensure the directory exists.
		$hSearch = FileFindFirstFile($g_aDirectories[$i][0] & "*.*")
		If $hSearch <> -1 Then
			FileClose($hSearch)
			; Contains advanced error information for files that fail syntax check.
			$bResult = CheckFiles($g_aDirectories[$i][0], $g_aDirectories[$i][1])
			$iNbErrors = @error
			$iNbFiles = @extended

			; Check the files and test if they were all successful.
			If $bResult Then
				; Output success message and increment success count.
				_OutputProgressWrite("(" & $iNbFiles & ") succeeded." & @CRLF)
				$nSuccessCount += 1
			Else
				; If showing the error message print verbose output, otherwise just print that it failed.
				If $iNbFiles = 0 Then
					_OutputProgressWrite("failed (No files to test)." & @CRLF)
					$nFailedCount += $iNbErrors
				Else
					$nFailedCount += $iNbErrors
					_OutputProgressWrite("(" & $iNbErrors & "/" & $iNbFiles & ") failed." & @CRLF)
				EndIf
			EndIf
		Else
			_OutputProgressWrite("failed (Directory does not exist)." & @CRLF)
		EndIf
	Next
	ToolTip("")

	; Wait for close (if applicable).
	If $nFailedCount Then
		_OutputProgressWrite("==== " & StringTrimRight(@ScriptName, StringLen(".exe")) & " Failed: " & $nDirectoryCount - $nSuccessCount & "/" & $nDirectoryCount & " (" & $nFailedCount & ") ====" & @CRLF & @CRLF)
	Else
		_OutputProgressWrite("==== " & StringTrimRight(@ScriptName, StringLen(".exe")) & " Completed: " & $nSuccessCount & "/" & $nDirectoryCount & " ====" & @CRLF & @CRLF)
	EndIf
	_OutputWaitClosed($nFailedCount)

	; Return 0 on success.
	Return $nFailedCount
EndFunc   ;==>_Main
#EndRegion _Main()

#Region Primary Functions

#Region CheckFiles()
; ===================================================================
; CheckFiles($sPath, $bAu3)
;
; Check Varname in the files in the specified directory.
; Parameters:
;	$sPath - IN - The input directory to search for files to test.
;	$bAu3 - IN - True is .au3 files to be checked.
; Returns:
;	Success - True
;	Failure - False.  May set @error as follows:
;		0 - No fatal errors occurred but some files did not pass syntax check.
;		1 - No files to process.
; ===================================================================
Func CheckFiles($sPath, $bAu3)
	Local $sExt = "*.txt"
	If $bAu3 Then $sExt = "*.au3"
	; Open a search handle to the specified directory.
	Local $hSearch = FileFindFirstFile($sPath & $sExt)

	; Ensure the search handle is valid.
	If $hSearch = -1 Then Return SetError(1, 0, False)

	; keep only Dirs
	$sPath = StringLeft($sPath, StringInStr($sPath, "\", $STR_NOCASESENSEBASIC, -1))

	; Stores the return value.  Assume true at start.
	Local $bResult = True
	Local $iNbErrors = 0, $iNbFiles = 0
	Local $sOutput, $sFile, $sFullPath, $iNbFoundErrors

	; Loop over the files.
	While True
		; Get the name of the next file.
		$sFile = FileFindNextFile($hSearch)

		; Test that the name is valid.
		If @error Then ExitLoop

		$iNbFiles += 1
		ToolTip($sFile, 200, 200)

		; Variable that stores the output from Au3Check.
		$sOutput = ""
		$g_sVarnameError = "???"

		; Build the path to the file.
		$sFullPath = $sPath & $sFile

		; Check if variable names follow standard convention
		$iNbFoundErrors = CheckVarNames($sFullPath, $bAu3, $sOutput)

		; Test the return value.
		If $iNbFoundErrors <> 0 Then
			; If any single file fails then the return value is false.
			$bResult = False
			$iNbErrors += 1

			; Append the error output to the error parameter.
			$sOutput = '"' & $sPath & $sFile & '" failed (' & $iNbFoundErrors & ") :" & @CRLF & @TAB & $sOutput
			_OutputBuildWrite($sOutput & @CRLF)
		EndIf
	WEnd

	; Close the search handle.
	FileClose($hSearch)

	Return SetError($iNbErrors, $iNbFiles, $bResult)
EndFunc   ;==>CheckFiles
#EndRegion CheckFiles()

#Region CheckVarNames()
; ===================================================================
; CheckVarNames($sFullPath, $bAu3, ByRef $sOutput)
;
; Check Variable Names in the specified file.
; Parameters:
;	$sFullPath - IN - The file to test.
;	$bAu3 - IN - Filename type : True for .au3 script file
;	$sOutput - OUT - Test output is captured to this variable.
; Returns:
;	The return nb of errors
; ===================================================================
Func CheckVarNames($sFullPath, $bAu3, ByRef $sOutput)
	Local $hIn = FileOpen($sFullPath)

	Local $sLine, $aVarNames
	Local $iNbErrors = 0, $sErrors

	While 1
		$sLine = GetVarNamesLine($hIn, $bAu3)
		If @error Then ExitLoop
		If GetVarNames($sLine, $aVarNames) Then
			$iNbErrors += CheckVarNamesErrors($aVarNames, $sErrors)
			; If StringInStr($sLine, "$tagSYMBOL_INFO ") Then ConsoleWrite('@@ Debug(' & @ScriptLineNumber & ') : $sLine = ' & $sLine & @CRLF & '>UBound($aVarNames): ' & UBound($aVarNames, 1) & '    $sErrors: ' & $sErrors & '    $g_bConst: ' & $g_bConst & @CRLF) ;### Debug Console
			If $sErrors Then
;~ 				_DebugArrayDisplay($aVarNames)
				ConsoleWrite('@@ Debug(' & @ScriptLineNumber & ') : $sLine = ' & $sLine & @CRLF & '>UBound($aVarNames): ' & UBound($aVarNames, 1) & '    $sErrors: ' & $sErrors & '    $g_bConst: ' & $g_bConst & @CRLF) ;### Debug Console
			EndIf
			$sOutput &= $sErrors
		EndIf
	WEnd

	FileClose($hIn)
	Return SetError(@error, @extended, $iNbErrors)
EndFunc   ;==>CheckVarNames
#EndRegion CheckVarNames()
; #include<Array.au3>
#Region GetVarNamesLine()
; ===================================================================
; GetVarNamesLine($hIn, $bAu3)
;
; Get Variable Names in the specified line.
; Parameters:
;	$hIn - IN - Handle of the file to be search
;	$bAu3 - IN - True if a .au3 file is being checked
; Returns:
;	String containing variable names
;	Set @error if End Of File has been reached
; ===================================================================
Func GetVarNamesLine($hIn, $bAu3)
	Local $sLine, $aMatch, $sTemp
	While 1
		$sLine = FileReadLine($hIn)
		If @error Then Return SetError(@error, @extended, "")

		; skip comments block
		If StringRegExp($sLine, '(?i)^\h*?#c(s|omment-start)') Then
			Do
				$sLine = FileReadLine($hIn)
				If @error Then Return SetError(@error, @extended, "")
			Until StringRegExp($sLine, '(?i)^\h*?#c(e|omment-end)')
			$sLine = FileReadLine($hIn)
			If @error Then Return SetError(@error, @extended, "")
		EndIf

		; skip comments line or directives
		If $bAu3 And StringRegExp($sLine, '^\h*?[;#]') Then ContinueLoop

		$g_bGlobal = False
		$g_bConst = False
		$g_bGUICtrlCreate = False

		If $bAu3 Then
			$sLine = StringRegExpReplace($sLine, '(\h;[^;]*$)', "") ; suppress trailing comment
			While StringRight($sLine, 1) = "_" ; merge continuation lines
				$sLine = StringTrimRight($sLine, 1)
				$sTemp = FileReadLine($hIn)
				$sLine &= StringRegExpReplace($sTemp, '(\h;[^;]*$)', "") ; suppress trailing comment
			WEnd

;~ 			If StringInStr($sLine, "$tagSYMBOL_INFO ") Then ConsoleWrite('@@ Debug(' & @ScriptLineNumber & ') : $sLine = ' & $sLine & @CRLF & '>Error code: ' & @error & '    Extended code: ' & @extended & ' (0x' & Hex(@extended) & ')' & @CRLF) ;### Debug Console
			$sLine = StringRegExpReplace($sLine, '([''"])\V*?\1', "0") ; Strip string literals. By PhoenixXL & guinness.
;~ 			If StringInStr($sLine, "$tagSYMBOL_INFO ") Then ConsoleWrite('@@ Debug(' & @ScriptLineNumber & ') : $sLine = ' & $sLine & @CRLF & '>Error code: ' & @error & '    Extended code: ' & @extended & ' (0x' & Hex(@extended) & ')' & @CRLF) ;### Debug Console

			If StringRegExp($sLine, '(?i)\h*=\h*GUICtrlCreate') Then
				$g_bGUICtrlCreate = True
				Return "(" & $sLine & " $)"
			EndIf

			$aMatch = StringRegExp($sLine, "(?i)^\h*\b(Global|Local|Func)\b", $STR_REGEXPARRAYMATCH)
			If @error Then ContinueLoop

			If $aMatch[0] <> "Func" Then
				; memorise Global Const type
				$g_bGlobal = ($aMatch[0] = "Global")
				$g_bConst = StringRegExp($sLine, '\b(Const|Enum)\b')

				; Add parenthesis and delimiter to ease extracting variable names
				$sLine = StringReplace($sLine, $aMatch[0], $aMatch[0] & " (", 1)
			Else
				$sLine = StringTrimRight($sLine, 1) ; strip ) to allow $ i nstertion at the end
			EndIf
			$sLine &= " $)"
			ExitLoop
		Else
			If StringInStr($sLine, "###Syntax###") Then
				; get Syntax definition
				$sLine = FileReadLine($hIn)
				If StringInStr($sLine, "#include") Then
					$sLine = FileReadLine($hIn)

					$sLine = StringRegExpReplace($sLine, '(\h*;[^;]*$)', "") ; suppress trailing comment
					While StringRight($sLine, 1) = "_" ; merge continuation lines
						$sLine = StringTrimRight($sLine, 1)
						$sTemp = FileReadLine($hIn)
						$sLine &= StringRegExpReplace($sTemp, '(\h*;[^;]*$)', "") ; suppress trailing comment
					WEnd

					$sLine = StringRegExpReplace($sLine, '([''"])\V*?\1', "0") ; Strip string literals. By PhoenixXL & guinness.
					$sLine = StringRegExpReplace($sLine, "(\[|\])", "") ; suppress []
				Else
					Return SetError(10, 0, "") ; the file is not OK for checking as for Function or $tag UDF help file
				EndIf
				$sLine = "Func " & StringRegExpReplace($sLine, "\)$", " $)") ; add an ending delimiter to ease extracting variable names
				ExitLoop
			EndIf
		EndIf
	WEnd

	; strip attributes
	$sLine = StringRegExpReplace($sLine, '(\b(Const|Static|ByRef|Enum)\b)|(\bStep\b\h*[\*\+-][1-9])', "")
	Return $sLine
EndFunc   ;==>GetVarNamesLine
#EndRegion GetVarNamesLine()

#Region CheckVarNamesErrors()
; ===================================================================
; CheckVarNamesErrors($aVarNames, ByRef $sOutput)
;
; Get Variable Names in the specified line.
; Parameters:
;	$aVarNames - In - array containing variable names.
;	$sOutput - OUT - Test output is captured to this variable.
; Returns:
;	Number of errors found
; ===================================================================
Func CheckVarNamesErrors($aVarNames, ByRef $sOutput)
	Local $bTypeOK, $j, $sVar, $sVarType, $iLen, $sTemp

	$g_iNbErrors = 0
	$sOutput = ""
	For $i = 0 To UBound($aVarNames, 1) - 1
		If ($i <= (UBound($aVarNames, 1) - 2)) And ($aVarNames[$i] = "=") Then
			$i += 2 ; skip assigned value
			If $i >= UBound($aVarNames, 1) Then ExitLoop
		EndIf

		$sVar = $aVarNames[$i]
		$j = 1
		Do
			$j += 1
		Until StringMid($sVar, $j, 1) <> "_" ; ignore leading '_'
		$sVar = StringMid($sVar, $j)
		$sVarType = StringLeft($sVar, 1)

		$j = 1
		If $g_bConst And $g_bGlobal Then
			If Not ($sVarType == "g") And Not ($sVarType == "e") Then
				; do not check $tag??? , $Wd??? , $mso??? or $xl??? if Global and Const
				If Not ((StringLeft($sVar, 3) == "tag") Or (StringLeft($sVar, 2) == "Wd") Or (StringLeft($sVar, 3) == "mso") Or (StringLeft($sVar, 2) == "xl")) Then
					;Check variable in UPPER case before first _ if any
					$sTemp = StringUpper($sVar)
					$j = StringInStr($sVar, "_")
					If $j = 0 Then $j = StringLen($sVar)
					If Not (StringLeft($sVar, $j) == StringLeft($sTemp, $j)) Then
						$sOutput &= OutputError($aVarNames[$i], $sVarType, "g", "GLOBAL")
					EndIf
				Else
					$j = 3
					If (StringLeft($sVar, 3) == "tag") Or (StringLeft($sVar, 3) == "mso") Then $j = 4
					If Not StringRegExp(StringMid($sVar, $j, 1), '[A-Z|0-9]') Then
						; an uppercase letter or a digit must follow type
						$sOutput &= OutputError($aVarNames[$i], StringMid($sVar, $j, 1), StringUpper(StringMid($sVar, $j, 1)), "")
					EndIf
				EndIf
				ContinueLoop
			EndIf
		Else
			$bTypeOK = True
			$iLen = StringLen($sVar)
			If $iLen = 1 Then ContinueLoop ; allow variable mono letter
			If $g_bGlobal Or $g_bGUICtrlCreate Then
				If ($sVarType == "g") Then
					$j += 1 ; skip global type
					If $iLen = ($j + 1) Then ContinueLoop ; allow global variable mono letter
					If StringMid($sVar, $j, 1) = "_" Then
						$j += 1 ; allow _ after g
					Else
						$sOutput &= OutputError($aVarNames[$i], StringMid($sVar, $j, 1), "_", "GLOBAL")
						ContinueLoop
					EndIf
				Else
					If Not $g_bGUICtrlCreate And Not ($sVar == StringUpper($sVar)) Then
						$sOutput &= OutputError($aVarNames[$i], $sVarType, "g", "GLOBAL")
						ContinueLoop
					EndIf
					$bTypeOK = $g_bGUICtrlCreate ; to force error if Global was used
				EndIf
			Else
				; cannot use StringIsUpper() as variable name may contain digit or _
				If $g_bConst And ($sVar == StringUpper($sVar)) Then ContinueLoop ; allow Const in Upper case
			EndIf

			; Check type
			$sVarType = StringMid($sVar, $j, 1)

			; Check if Array variable as a valid type
			If $sVarType == "a" Or $sVarType == "m" Or $sVarType == "v" Then
				If StringMid($sVar, $j + 1, 1) = "_" Then $j += 1 ; allow _ after a
				; Array type optional
				If StringRegExp(StringMid($sVar, $j + 1, 1), '[dhibfnsvopt]') Then
					$sVarType = StringMid($sVar, $j + 1, 1)
					$j += 1
				EndIf
			Else
				If StringInStr($aVarNames[$i], "[") Then
					$sOutput &= OutputError($aVarNames[$i], $sVarType, "a/m/v", "ARRAY/MAP")
					ContinueLoop
				EndIf
			EndIf

			If StringIsUpper($sVarType) Then
				$bTypeOK = False ; to force error
			Else
				; Check if assignment variable as a valid type
				If $i < (UBound($aVarNames, 1) - 2) Then
					If ($aVarNames[$i + 1] = "=") Then
						; Check if Boolean variable as a valid type
						If ($aVarNames[$i + 2] = "True" Or $aVarNames[$i + 2] = "False") Then
							If ($sVarType <> "b") And Not $g_bConst Then ; Constant name are not checked
								$sOutput &= OutputError($aVarNames[$i], $sVarType, "b", "BOOLEAN")
								ContinueLoop
							EndIf
						ElseIf StringLeft($aVarNames[$i + 2], 13) = "GUICtrlCreate" Then
							If Not (StringMid($sVar, $j, 2) == "id") And Not ($sVarType == "v") Then
								$sOutput &= OutputError($aVarNames[$i], $sVarType, "id", "CTRL")
								ContinueLoop
							EndIf
						ElseIf StringLeft($aVarNames[$i + 2], 15) = "DLLStructCreate" Then
							If $sVarType <> "t" Then
								$sOutput &= OutputError($aVarNames[$i], $sVarType, "t", "DLLSTRUCT")
								ContinueLoop
							EndIf
						EndIf
					EndIf
				EndIf

				Switch $sVarType
					Case "i"
						; AutoIt Control ID
						If StringMid($sVar, $j + 1, 1) == "d" Then
							$sVarType = "id"
							$j += 1
						EndIf
						If (StringMid($sVar, $j + 1, 1) == "w") Or (StringMid($sVar, $j + 1, 1) == "l") Then
							$j += 1
							ContinueCase
						EndIf

					Case "w", "l"
						; w and l type are dummy type for wParam or lParam
						If StringMid($sVar, $j + 1, 5) == "Param" Then ContinueLoop

					Case "p"
						If StringMid($sVar, $j + 1, 1) = "_" Then $j += 1 ; allow _ after p
						; Ptr type optional
						If StringRegExp(StringMid($sVar, $j + 1, 1), '[adhibfnsvo]') Then
							$sVarType = StringMid($sVar, $j + 1, 1)
							$j += 1
						EndIf

					Case "t"
						If StringMid($sVar, $j + 1, 2) == "ag" Then
							; Structure string definition
							$sVarType = "tag"
							$j += 2
						EndIf

					Case "f"
						If StringMid($sVar, $j + 1, 1) == "u" Then
							; Function ref
							$sVarType = "fu"
							$j += 1
						EndIf

					Case "a", "m", "d", "h", "b", "n", "s", "v", "o"
						; just valid type

					Case "0" To "9"
						If StringIsDigit(StringTrimLeft($sVar, 1)) Then ContinueLoop ; $dd is a valid name

					Case "e"
						; Enum/Const variable type
						If $g_bConst Then ContinueLoop
						$bTypeOK = False ; to force error

					Case Else
						$bTypeOK = False ; to force error
				EndSwitch
			EndIf

			If $bTypeOK Then
				$j += 1
				If StringMid($sVar, $j, 1) <> "[" Then
					; not an array with single letter $a(.]
					If StringMid($sVar, $j, 1) = "_" Then $j += 1 ; allow _ after type
					If Not StringRegExp(StringMid($sVar, $j, 1), '[A-Z]|[0-9]') Then
						; an uppercase letter or a digit must follow type
						$sOutput &= OutputError($aVarNames[$i], StringMid($sVar, $j, 1), StringUpper(StringMid($sVar, $j, 1)), "")
					EndIf
				EndIf
			Else
				$sOutput &= OutputError($aVarNames[$i], StringMid($sVar, $j, 1), "?", "?")
			EndIf
		EndIf
	Next

	Return $g_iNbErrors
EndFunc   ;==>CheckVarNamesErrors
#EndRegion CheckVarNamesErrors()

#Region OutputError()
; ===================================================================
; OutputError($sVarname, $sVartype, $sErrorType, $sErrorMsg)
;
; Get Variable Names in the specified line.
; Parameters:
;	$sVarname - IN - variable name
;	$sVartype - IN - type of the variable
;	$sErrorType - IN - type error
;	$sErrorMsg - IN - type error message
; Returns:
;	String message
; ===================================================================
Func OutputError($sVarname, $sVarType, $sErrorType, $sErrorMsg)
	$g_iNbErrors += 1
	If $g_sVarnameError = $sVarname Then Return ""

	$g_sVarnameError = $sVarname
	If $sErrorMsg = '?' Then Return $sVarname & " [ " & $sErrorType & $sVarType & " ] "
	Return $sVarname & " [ " & $sVarType & " -> (" & $sErrorType & ") " & $sErrorMsg & " ] "
EndFunc   ;==>OutputError
#EndRegion OutputError()

#Region GetVarNames()
; ===================================================================
; GetVarNames($sLine, $bAu3, ByRef $aVarNames)
;
; Get Variable Names in the specified line.
; Parameters:
;	$sLine - IN - file line to be checked
;	$aVarNames - OUT - array containing variable names.
; Returns:
;	True if variables are defined
; ===================================================================
Func GetVarNames($sLine, ByRef $aVarNames)
	$aVarNames = StringRegExp($sLine, '\((.*?)\)$', $STR_REGEXPARRAYMATCH) ; suppress func name/Local/Global

	$aVarNames = StringRegExpReplace($aVarNames[0], '(\(([^()]++|(?R))*\))', " 0") ; suppress recursive parenthensis to squeeze func parameter

	$aVarNames = StringRegExpReplace($aVarNames, '(\[([^\[\]]++|(?R))*\])', "[.]") ; suppress recursive bracket
	$aVarNames = StringRegExpReplace($aVarNames, '=\h*\[\.\]', "= 0") ; suppress array assignment
;~ 	$aVarNames = StringRegExpReplace($aVarNames, '&.*&', "&") ; suppress internal concatanation
	$aVarNames = StringReplace($aVarNames, ': $', "") ; suppress Ternary assignment

	; create array of parameters with assign value. By jchd.
;~ 	$aVarNames = StringRegExp($aVarNames, '(?x) ( \$ \w+ (?: \[ [^\]]* \] )* ) (?: \h* (=) \h* ( [-]? [\$|@|.]? \w+ )? .*? [,|$] )?', $STR_REGEXPARRAYGLOBALMATCH)
	$aVarNames = StringRegExp($aVarNames, '(?x) ( \$ \w+ (?: \[ [^\]]* \] )* ) (?: \h* (=) \h* ( [-]? [\$|@|.]? \w+ )? .*? (?:,|$) )?', $STR_REGEXPARRAYGLOBALMATCH)

	Return IsArray($aVarNames)
EndFunc   ;==>GetVarNames
#EndRegion GetVarNames()

#EndRegion Primary Functions
