#include '..\..\..\..\..\_build\include\MiscLib.au3'
#include '..\gen_editorfuncs.au3'

GenerateSyntax()

; ------------------------------------------------------------------------------
; Automatically generate the syntax file(s).
; ------------------------------------------------------------------------------
Func GenerateSyntax()

	; Create Notepad++ keyword & macro list.
	Local $sKeywordAndMacroList = GetKeywordsAndMacros()

	; Create Notepad++ editor list.
	Local $sFunctionList = "<?xml version='1.0' encoding='Windows-1252' ?>" & @CRLF & _
			"<NotepadPlus>" & @CRLF & _
			@TAB & "<AutoComplete language='AutoIt'>" & @CRLF & _
			@TAB & @TAB & "<Environment ignoreCase='yes' startFunc='(' stopFunc=')' paramSeparator=',' terminal='' />" & @CRLF & _
			$sKeywordAndMacroList & _
			GetFunctions() & _
			@TAB & "</AutoComplete>" & @CRLF & _
			"</NotepadPlus>"

	Return _FileWrite(@ScriptDir & '\autoit.xml', $sFunctionList)
EndFunc   ;==>GenerateSyntax

; ------------------------------------------------------------------------------
; Retrieve the keywords and macros.
; ------------------------------------------------------------------------------
Func GetKeywordsAndMacros()
	Local $sData = FileRead($KEYWORDLIST) & FileRead($MACROLIST)
	Local $aKeywordsAndMacros = StringSplit($sData, @CRLF, BitOR($STR_ENTIRESPLIT, $STR_NOCOUNT)), _
			$sKeywordAndMacroList = ''
	For $i = 0 To UBound($aKeywordsAndMacros) - 1
		$sKeywordAndMacroList &= @TAB & @TAB & "<KeyWord name='" & $aKeywordsAndMacros[$i] & "' />" & @CRLF
	Next
	Return $sKeywordAndMacroList
EndFunc   ;==>GetKeywordsAndMacros

; ------------------------------------------------------------------------------
; Retrieve the functions, syntax and description.
; ------------------------------------------------------------------------------
Func GetFunctions()
	Local $sData = FileRead($FUNCTIONLIST_FULL) & $ASCII_ACK & @CRLF & FileRead($LIBFUNCTIONLIST_FULL) & @CRLF
	Local $aFunctions = StringSplit($sData, $ASCII_ACK, $STR_NOCOUNT), $aSplit = 0, _
			$sDescription = '', $sFunctionName = '', $sFunctionList = '', $sParamList = ''

	Local $iUBound = UBound($aFunctions)
	If $iUBound >= $FUNCTIONLIST_MAX Then
		For $i = 0 To $iUBound - 1 Step 3
			$sFunctionName = $aFunctions[$i + $FUNCTIONLIST_NAME]

			$sParamList = StringStripWS($aFunctions[$i + $FUNCTIONLIST_SYNTAX], BitOR($STR_STRIPLEADING, $STR_STRIPTRAILING))
			_ConvertSymbolsToHTMLEntity($sParamList)
			$aSplit = StringRegExp($sParamList, '\w+\h*\((.*?)\h*\)', $STR_REGEXPARRAYGLOBALMATCH)
			If UBound($aSplit) > 0 Then $sParamList = $aSplit[0]

			$sDescription = $aFunctions[$i + $FUNCTIONLIST_DESCRIPTION]
			_ConvertSymbolsToHTMLEntity($sDescription)
			$sDescription = InsertLineBreak($sDescription)
			; Quick workaround.
			; $sDescription = StringReplace($sDescription,  ' &#x0a; ', '&#x0a;')
			$sDescription = StringStripWS($sDescription, $STR_STRIPSPACES)

			$sFunctionName = StringStripWS($sFunctionName, $STR_STRIPALL)
			$sFunctionList &= @TAB & @TAB & "<KeyWord name='" & $sFunctionName & "' func='yes' >" & @CRLF
			$sFunctionList &= @TAB & @TAB & @TAB & "<Overload retVal='' descr='" & $sDescription & "' " & ($sParamList ? "" : "/") & ">" & @CRLF

			If $sParamList <> '' Then
				$aSplit = StringSplit($sParamList, ',', $STR_NOCOUNT)
				For $j = 0 To UBound($aSplit) - 1
					$sFunctionList &= @TAB & @TAB & @TAB & @TAB & "<Param name='" & StringStripWS($aSplit[$j], $STR_STRIPLEADING) & "' />" & @CRLF
				Next
				$sFunctionList &= @TAB & @TAB & @TAB & '</Overload>' & @CRLF
			EndIf

			$sFunctionList &= @TAB & @TAB & '</KeyWord>' & @CRLF
		Next
	EndIf

	Return $sFunctionList
EndFunc   ;==>GetFunctions

; ------------------------------------------------------------------------------
; Insert breaks every 100 characters.
; ------------------------------------------------------------------------------
Func InsertLineBreak($sData) ; Add escape breaks. Idea by jaberwocky6669 and guinness.
	Local Const $iBreak_Point = 100
	Local Const $iLength = StringLen($sData)
	If $iLength < $iBreak_Point Then Return $sData

	Local $bIsBreakPoint = False, _
			$iStart = $iBreak_Point, _
			$sChr = '', $sLine = ''
	Local Const $NP_NL = ' &#x0a; ', _
			$STR_SPACE = Chr(32)
	For $i = 1 To $iLength
		$sChr = StringMid($sData, $i, 1)

		If $i = $iStart Or $bIsBreakPoint Then
			If $sChr = $STR_SPACE Then
				$bIsBreakPoint = False
				$iStart += $iBreak_Point
				$sChr &= $NP_NL
			Else
				$bIsBreakPoint = True
				$iStart += 1
			EndIf
		EndIf
		$sLine &= $sChr
	Next
	Return $sLine
EndFunc   ;==>InsertLineBreak
