#include-once

#include <FileConstants.au3>
#include <StringConstants.au3>

#include '..\gen_editorfuncs.au3'

; ------------------------------------------------------------------------------
; Automatically generate the definition syntax file(s) for functions, keywords and macros.
; ------------------------------------------------------------------------------
Func GenerateDefSyntax()
	If Not FileExists($FUNCTIONLIST_FULL) Or Not FileExists($MACROLIST_FULL) Then Return False

	; Get the Functions list.
	Local $sFunctionList = GetFunctions()

	; Get the Macros list.
	Local $sMacroList = GetMacros()

	FileCopy(@ScriptDir & '\extra.def', @ScriptDir & '\AutoIt3.def', $FC_OVERWRITE)
	Local $hFileOpen = FileOpen(@ScriptDir & '\AutoIt3.def', $FO_APPEND)
	If $hFileOpen > -1 Then
		FileWriteLine($hFileOpen, $sFunctionList & $sMacroList)
		FileClose($hFileOpen)
	EndIf

	Return True
EndFunc   ;==>GenerateDefSyntax

; ------------------------------------------------------------------------------
; Retrieve the functions, syntax and description.
; ------------------------------------------------------------------------------
Func GetFunctions()
	Local $sData = FileRead($FUNCTIONLIST_FULL) & $ASCII_ACK & @CRLF & _
			FileRead($LIBFUNCTIONLIST_FULL) & @CRLF
	Local $aFunctions = StringSplit($sData, $ASCII_ACK, $STR_NOCOUNT), _
			$sFunctionList = ''
	Local $iUBound = UBound($aFunctions)
	If $iUBound >= $FUNCTIONLIST_MAX Then
		For $i = 0 To UBound($aFunctions) - 1 Step 3
			$aFunctions[$i] = StringStripWS($aFunctions[$i + $FUNCTIONLIST_NAME], $STR_STRIPALL)
			$sFunctionList &= '[' & $aFunctions[$i + $FUNCTIONLIST_NAME] & ' | ' & $aFunctions[$i + $FUNCTIONLIST_DESCRIPTION] & ']' & @CRLF
			$sFunctionList &= StringReplace($aFunctions[$i + $FUNCTIONLIST_SYNTAX], '( ', '( |') & @CRLF
			$sFunctionList &= ';' & @CRLF
		Next
	EndIf
	Return $sFunctionList
EndFunc   ;==>GetFunctions

; ------------------------------------------------------------------------------
; Retrieve the list of macros and their description.
; ------------------------------------------------------------------------------
Func GetMacros()
	Local Enum $eMACRO_NAME, $eMACRO_DESCRIPTION
	Local $sData = FileRead($MACROLIST_FULL)
	Local $aMacros = StringSplit($sData, $ASCII_ACK, $STR_NOCOUNT), _
			$sMacroList = ''
	For $i = 0 To UBound($aMacros) - 1 Step 2
		$aMacros[$i] = StringStripWS($aMacros[$i], $STR_STRIPALL)
		$sMacroList &= '[' & $aMacros[$i + $eMACRO_NAME] & ' | ' & $aMacros[$i + $eMACRO_DESCRIPTION] & ']' & @CRLF
		$sMacroList &= $aMacros[$i] & @CRLF
		$sMacroList &= ';' & @CRLF
	Next
	Return $sMacroList
EndFunc   ;==>GetMacros
